"""Base resource class for Contiss API resources."""

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from .._http import HTTPClient


class BaseResource:
    """Base class for all API resources."""

    def __init__(self, client: "HTTPClient") -> None:
        """Initialize the resource.

        Args:
            client: HTTP client instance
        """
        self._client = client
